@@ -2,11 +2,11 @@
 
 use crate::{
     common::{Gid, Uid},
-    sys::{Component, Disk, Networks, Process, Processor},
+    sys::{Component, Cpu, Disk, Networks, Process},
 };
 use crate::{
-    DiskType, DiskUsage, LoadAvg, NetworksIter, Pid, ProcessRefreshKind, ProcessStatus,
-    RefreshKind, Signal, User,
+    CpuRefreshKind, DiskType, DiskUsage, LoadAvg, NetworksIter, Pid, ProcessRefreshKind,
+    ProcessStatus, RefreshKind, Signal, User,
 };
 
 use std::collections::HashMap;
@@ -208,6 +208,18 @@ pub trait ProcessExt: Debug {
     ///     println!("{}", process.exe().display());
     /// }
     /// ```
+    ///
+    /// ### Implementation notes
+    ///
+    /// On Linux, this method will return an empty path if there
+    /// was an error trying to read `/proc/<pid>/exe`. This can
+    /// happen, for example, if the permission levels or UID namespaces
+    /// between the caller and target processes are different.
+    ///
+    /// It is also the case that `cmd[0]` is _not_ usually a correct
+    /// replacement for this.
+    /// A process [may change its `cmd[0]` value](https://man7.org/linux/man-pages/man5/proc.5.html)
+    /// freely, making this an untrustworthy source of information.
     fn exe(&self) -> &Path;
 
     /// Returns the pid of the process.
@@ -258,26 +270,26 @@ pub trait ProcessExt: Debug {
     /// ```
     fn root(&self) -> &Path;
 
-    /// Returns the memory usage (in KB).
+    /// Returns the memory usage (in bytes).
     ///
     /// ```no_run
     /// use sysinfo::{Pid, ProcessExt, System, SystemExt};
     ///
     /// let s = System::new();
     /// if let Some(process) = s.process(Pid::from(1337)) {
-    ///     println!("{} KB", process.memory());
+    ///     println!("{} bytes", process.memory());
     /// }
     /// ```
     fn memory(&self) -> u64;
 
-    /// Returns the virtual memory usage (in KB).
+    /// Returns the virtual memory usage (in bytes).
     ///
     /// ```no_run
     /// use sysinfo::{Pid, ProcessExt, System, SystemExt};
     ///
     /// let s = System::new();
     /// if let Some(process) = s.process(Pid::from(1337)) {
-    ///     println!("{} KB", process.virtual_memory());
+    ///     println!("{} bytes", process.virtual_memory());
     /// }
     /// ```
     fn virtual_memory(&self) -> u64;
@@ -334,7 +346,7 @@ pub trait ProcessExt: Debug {
     /// multicore machine.
     ///
     /// If you want a value between 0% and 100%, divide the returned value by the number of CPU
-    /// processors.
+    /// CPUs.
     ///
     /// **Warning**: If you want accurate CPU usage number, better leave a bit of time
     /// between two calls of this method (200 ms for example).
@@ -370,69 +382,129 @@ pub trait ProcessExt: Debug {
     /// }
     /// ```
     fn disk_usage(&self) -> DiskUsage;
+
+    /// Returns the ID of the owner user of this process or `None` if this information couldn't
+    /// be retrieved. If you want to get the [`User`] from it, take a look at
+    /// [`SystemExt::get_user_by_id`].
+    ///
+    /// ```no_run
+    /// use sysinfo::{Pid, ProcessExt, System, SystemExt};
+    ///
+    /// let mut s = System::new_all();
+    ///
+    /// if let Some(process) = s.process(Pid::from(1337)) {
+    ///     eprintln!("User id for process 1337: {:?}", process.user_id());
+    /// }
+    /// ```
+    fn user_id(&self) -> Option<&Uid>;
+
+    /// Returns the process group ID of the process.
+    ///
+    /// ⚠️ It always returns `None` on Windows.
+    ///
+    /// ```no_run
+    /// use sysinfo::{Pid, ProcessExt, System, SystemExt};
+    ///
+    /// let mut s = System::new_all();
+    ///
+    /// if let Some(process) = s.process(Pid::from(1337)) {
+    ///     eprintln!("Group ID for process 1337: {:?}", process.group_id());
+    /// }
+    /// ```
+    fn group_id(&self) -> Option<Gid>;
+
+    /// Wait for process termination.
+    ///
+    /// ```no_run
+    /// use sysinfo::{Pid, ProcessExt, System, SystemExt};
+    ///
+    /// let mut s = System::new_all();
+    ///
+    /// if let Some(process) = s.process(Pid::from(1337)) {
+    ///     eprintln!("Waiting for pid 1337");
+    ///     process.wait();
+    ///     eprintln!("Pid 1337 exited");
+    /// }
+    /// ```
+    fn wait(&self);
+
+    /// Returns the session ID for the current process or `None` if it couldn't be retrieved.
+    ///
+    /// ⚠️ This information is computed every time this method is called.
+    ///
+    /// ```no_run
+    /// use sysinfo::{Pid, ProcessExt, System, SystemExt};
+    ///
+    /// let mut s = System::new_all();
+    ///
+    /// if let Some(process) = s.process(Pid::from(1337)) {
+    ///     eprintln!("Session ID for process 1337: {:?}", process.session_id());
+    /// }
+    /// ```
+    fn session_id(&self) -> Option<Pid>;
 }
 
-/// Contains all the methods of the [`Processor`][crate::Processor] struct.
-pub trait ProcessorExt: Debug {
-    /// Returns this processor's usage.
+/// Contains all the methods of the [`Cpu`][crate::Cpu] struct.
+pub trait CpuExt: Debug {
+    /// Returns this CPU's usage.
     ///
     /// Note: You'll need to refresh it at least twice (diff between the first and the second is
     /// how CPU usage is computed) at first if you want to have a non-zero value.
     ///
     /// ```no_run
-    /// use sysinfo::{ProcessorExt, System, SystemExt};
+    /// use sysinfo::{CpuExt, System, SystemExt};
     ///
     /// let s = System::new();
-    /// for processor in s.processors() {
-    ///     println!("{}%", processor.cpu_usage());
+    /// for cpu in s.cpus() {
+    ///     println!("{}%", cpu.cpu_usage());
     /// }
     /// ```
     fn cpu_usage(&self) -> f32;
 
-    /// Returns this processor's name.
+    /// Returns this CPU's name.
     ///
     /// ```no_run
-    /// use sysinfo::{ProcessorExt, System, SystemExt};
+    /// use sysinfo::{CpuExt, System, SystemExt};
     ///
     /// let s = System::new();
-    /// for processor in s.processors() {
-    ///     println!("{}", processor.name());
+    /// for cpu in s.cpus() {
+    ///     println!("{}", cpu.name());
     /// }
     /// ```
     fn name(&self) -> &str;
 
-    /// Returns the processor's vendor id.
+    /// Returns the CPU's vendor id.
     ///
     /// ```no_run
-    /// use sysinfo::{ProcessorExt, System, SystemExt};
+    /// use sysinfo::{CpuExt, System, SystemExt};
     ///
     /// let s = System::new();
-    /// for processor in s.processors() {
-    ///     println!("{}", processor.vendor_id());
+    /// for cpu in s.cpus() {
+    ///     println!("{}", cpu.vendor_id());
     /// }
     /// ```
     fn vendor_id(&self) -> &str;
 
-    /// Returns the processor's brand.
+    /// Returns the CPU's brand.
     ///
     /// ```no_run
-    /// use sysinfo::{ProcessorExt, System, SystemExt};
+    /// use sysinfo::{CpuExt, System, SystemExt};
     ///
     /// let s = System::new();
-    /// for processor in s.processors() {
-    ///     println!("{}", processor.brand());
+    /// for cpu in s.cpus() {
+    ///     println!("{}", cpu.brand());
     /// }
     /// ```
     fn brand(&self) -> &str;
 
-    /// Returns the processor's frequency.
+    /// Returns the CPU's frequency.
     ///
     /// ```no_run
-    /// use sysinfo::{ProcessorExt, System, SystemExt};
+    /// use sysinfo::{CpuExt, System, SystemExt};
     ///
     /// let s = System::new();
-    /// for processor in s.processors() {
-    ///     println!("{}", processor.frequency());
+    /// for cpu in s.cpus() {
+    ///     println!("{}", cpu.frequency());
     /// }
     /// ```
     fn frequency(&self) -> u64;
@@ -464,7 +536,7 @@ pub trait SystemExt: Sized + Debug + Default + Send + Sync {
     /// ```
     const SUPPORTED_SIGNALS: &'static [Signal];
 
-    /// Creates a new [`System`] instance with nothing loaded except the processors list. If you
+    /// Creates a new [`System`] instance with nothing loaded except the cpus list. If you
     /// want to load components, network interfaces or the disks, you'll have to use the
     /// `refresh_*_list` methods. [`SystemExt::refresh_networks_list`] for example.
     ///
@@ -538,8 +610,8 @@ pub trait SystemExt: Sized + Debug + Default + Send + Sync {
         if refreshes.memory() {
             self.refresh_memory();
         }
-        if refreshes.cpu() {
-            self.refresh_cpu();
+        if let Some(kind) = refreshes.cpu() {
+            self.refresh_cpu_specifics(kind);
         }
         if refreshes.components_list() {
             self.refresh_components_list();
@@ -564,6 +636,24 @@ pub trait SystemExt: Sized + Debug + Default + Send + Sync {
         }
     }
 
+    /// Refreshes all system, processes, disks and network interfaces information.
+    ///
+    /// Please note that it doesn't recompute disks list, components list, network interfaces
+    /// list nor users list.
+    ///
+    /// ```no_run
+    /// use sysinfo::{System, SystemExt};
+    ///
+    /// let mut s = System::new_all();
+    /// s.refresh_all();
+    /// ```
+    fn refresh_all(&mut self) {
+        self.refresh_system();
+        self.refresh_processes();
+        self.refresh_disks();
+        self.refresh_networks();
+    }
+
     /// Refreshes system information (RAM, swap, CPU usage and components' temperature).
     ///
     /// If you want some more specific refreshes, you might be interested into looking at
@@ -595,19 +685,37 @@ pub trait SystemExt: Sized + Debug + Default + Send + Sync {
     /// ```
     fn refresh_memory(&mut self);
 
-    /// Refreshes CPU usage.
+    /// Refreshes CPUs information.
     ///
     /// ⚠️ Please note that the result will very likely be inaccurate at the first call.
     /// You need to call this method at least twice (with a bit of time between each call, like
     /// 200ms) to get accurate values as it uses previous results to compute the next value.
     ///
+    /// Calling this method is the same as calling
+    /// `refresh_cpu_specifics(CpuRefreshKind::new().with_cpu_usage())`.
+    ///
     /// ```no_run
     /// use sysinfo::{System, SystemExt};
     ///
     /// let mut s = System::new_all();
     /// s.refresh_cpu();
     /// ```
-    fn refresh_cpu(&mut self);
+    fn refresh_cpu(&mut self) {
+        self.refresh_cpu_specifics(CpuRefreshKind::new().with_cpu_usage())
+    }
+
+    /// Refreshes CPUs specific information.
+    ///
+    /// Please note that it doesn't recompute disks list, components list, network interfaces
+    /// list nor users list.
+    ///
+    /// ```no_run
+    /// use sysinfo::{System, SystemExt, CpuRefreshKind};
+    ///
+    /// let mut s = System::new_all();
+    /// s.refresh_cpu_specifics(CpuRefreshKind::everything());
+    /// ```
+    fn refresh_cpu_specifics(&mut self, refresh_kind: CpuRefreshKind);
 
     /// Refreshes components' temperature.
     ///
@@ -708,6 +816,16 @@ pub trait SystemExt: Sized + Debug + Default + Send + Sync {
 
     /// The disk list will be emptied then completely recomputed.
     ///
+    /// ## Linux
+    ///
+    /// ⚠️ On linux, the [NFS](https://en.wikipedia.org/wiki/Network_File_System) file
+    /// systems are ignored and the information of a mounted NFS **cannot** be obtained
+    /// via [`SystemExt::refresh_disks_list`]. This is due to the fact that I/O function
+    /// `statvfs` used by [`SystemExt::refresh_disks_list`] is blocking and
+    /// [may hang](https://github.com/GuillaumeGomez/sysinfo/pull/876) in some cases,
+    /// requiring to call `systemctl stop` to terminate the NFS service from the remote
+    /// server in some cases.
+    ///
     /// ```no_run
     /// use sysinfo::{System, SystemExt};
     ///
@@ -771,24 +889,6 @@ pub trait SystemExt: Sized + Debug + Default + Send + Sync {
         self.networks_mut().refresh_networks_list();
     }
 
-    /// Refreshes all system, processes, disks and network interfaces information.
-    ///
-    /// Please note that it doesn't recompute disks list, components list, network interfaces
-    /// list nor users list.
-    ///
-    /// ```no_run
-    /// use sysinfo::{System, SystemExt};
-    ///
-    /// let mut s = System::new_all();
-    /// s.refresh_all();
-    /// ```
-    fn refresh_all(&mut self) {
-        self.refresh_system();
-        self.refresh_processes();
-        self.refresh_disks();
-        self.refresh_networks();
-    }
-
     /// Returns the process list.
     ///
     /// ```no_run
@@ -818,6 +918,12 @@ pub trait SystemExt: Sized + Debug + Default + Send + Sync {
     /// If you want only the processes with exactly the given `name`, take a look at
     /// [`SystemExt::processes_by_exact_name`].
     ///
+    /// **⚠️ Important ⚠️**
+    ///
+    /// On **linux**, there are two things to know about processes' name:
+    ///  1. It is limited to 15 characters.
+    ///  2. It is not always the exe name.
+    ///
     /// ```no_run
     /// use sysinfo::{ProcessExt, System, SystemExt};
     ///
@@ -843,6 +949,12 @@ pub trait SystemExt: Sized + Debug + Default + Send + Sync {
     /// If you instead want the processes containing `name`, take a look at
     /// [`SystemExt::processes_by_name`].
     ///
+    /// **⚠️ Important ⚠️**
+    ///
+    /// On **linux**, there are two things to know about processes' name:
+    ///  1. It is limited to 15 characters.
+    ///  2. It is not always the exe name.
+    ///
     /// ```no_run
     /// use sysinfo::{ProcessExt, System, SystemExt};
     ///
@@ -863,59 +975,63 @@ pub trait SystemExt: Sized + Debug + Default + Send + Sync {
         )
     }
 
-    /// Returns "global" processors information (aka the addition of all the processors).
+    /// Returns "global" cpus information (aka the addition of all the CPUs).
     ///
     /// To have up-to-date information, you need to call [`SystemExt::refresh_cpu`] or
     /// [`SystemExt::refresh_specifics`] with `cpu` enabled.
     ///
     /// ```no_run
-    /// use sysinfo::{ProcessorExt, RefreshKind, System, SystemExt};
+    /// use sysinfo::{CpuRefreshKind, CpuExt, RefreshKind, System, SystemExt};
     ///
-    /// let s = System::new_with_specifics(RefreshKind::new().with_cpu());
-    /// println!("{}%", s.global_processor_info().cpu_usage());
+    /// let s = System::new_with_specifics(
+    ///     RefreshKind::new().with_cpu(CpuRefreshKind::everything()),
+    /// );
+    /// println!("{}%", s.global_cpu_info().cpu_usage());
     /// ```
-    fn global_processor_info(&self) -> &Processor;
+    fn global_cpu_info(&self) -> &Cpu;
 
-    /// Returns the list of the processors.
+    /// Returns the list of the CPUs.
     ///
-    /// By default, the list of processors is empty until you call [`SystemExt::refresh_cpu`] or
+    /// By default, the list of cpus is empty until you call [`SystemExt::refresh_cpu`] or
     /// [`SystemExt::refresh_specifics`] with `cpu` enabled.
     ///
     /// ```no_run
-    /// use sysinfo::{ProcessorExt, RefreshKind, System, SystemExt};
+    /// use sysinfo::{CpuRefreshKind, CpuExt, RefreshKind, System, SystemExt};
     ///
-    /// let s = System::new_with_specifics(RefreshKind::new().with_cpu());
-    /// for processor in s.processors() {
-    ///     println!("{}%", processor.cpu_usage());
+    /// let s = System::new_with_specifics(
+    ///     RefreshKind::new().with_cpu(CpuRefreshKind::everything()),
+    /// );
+    /// for cpu in s.cpus() {
+    ///     println!("{}%", cpu.cpu_usage());
     /// }
     /// ```
-    fn processors(&self) -> &[Processor];
+    fn cpus(&self) -> &[Cpu];
 
-    /// Returns the number of physical cores on the processor or `None` if it couldn't get it.
+    /// Returns the number of physical cores on the CPU or `None` if it couldn't get it.
     ///
     /// In case there are multiple CPUs, it will combine the physical core count of all the CPUs.
     ///
     /// **Important**: this information is computed every time this function is called.
     ///
     /// ```no_run
-    /// use sysinfo::{ProcessorExt, System, SystemExt};
+    /// use sysinfo::{CpuExt, System, SystemExt};
     ///
     /// let s = System::new();
     /// println!("{:?}", s.physical_core_count());
     /// ```
     fn physical_core_count(&self) -> Option<usize>;
 
-    /// Returns the RAM size in KB.
+    /// Returns the RAM size in bytes.
     ///
     /// ```no_run
     /// use sysinfo::{System, SystemExt};
     ///
     /// let s = System::new_all();
-    /// println!("{} KB", s.total_memory());
+    /// println!("{} bytes", s.total_memory());
     /// ```
     fn total_memory(&self) -> u64;
 
-    /// Returns the amount of free RAM in KB.
+    /// Returns the amount of free RAM in bytes.
     ///
     /// Generally, "free" memory refers to unallocated memory whereas "available" memory refers to
     /// memory that is available for (re)use.
@@ -927,11 +1043,11 @@ pub trait SystemExt: Sized + Debug + Default + Send + Sync {
     /// use sysinfo::{System, SystemExt};
     ///
     /// let s = System::new_all();
-    /// println!("{} KB", s.free_memory());
+    /// println!("{} bytes", s.free_memory());
     /// ```
     fn free_memory(&self) -> u64;
 
-    /// Returns the amount of available RAM in KB.
+    /// Returns the amount of available RAM in bytes.
     ///
     /// Generally, "free" memory refers to unallocated memory whereas "available" memory refers to
     /// memory that is available for (re)use.
@@ -943,47 +1059,47 @@ pub trait SystemExt: Sized + Debug + Default + Send + Sync {
     /// use sysinfo::{System, SystemExt};
     ///
     /// let s = System::new_all();
-    /// println!("{} KB", s.available_memory());
+    /// println!("{} bytes", s.available_memory());
     /// ```
     fn available_memory(&self) -> u64;
 
-    /// Returns the amound of used RAM in KB.
+    /// Returns the amount of used RAM in bytes.
     ///
     /// ```no_run
     /// use sysinfo::{System, SystemExt};
     ///
     /// let s = System::new_all();
-    /// println!("{} KB", s.used_memory());
+    /// println!("{} bytes", s.used_memory());
     /// ```
     fn used_memory(&self) -> u64;
 
-    /// Returns the SWAP size in KB.
+    /// Returns the SWAP size in bytes.
     ///
     /// ```no_run
     /// use sysinfo::{System, SystemExt};
     ///
     /// let s = System::new_all();
-    /// println!("{} KB", s.total_swap());
+    /// println!("{} bytes", s.total_swap());
     /// ```
     fn total_swap(&self) -> u64;
 
-    /// Returns the amount of free SWAP in KB.
+    /// Returns the amount of free SWAP in bytes.
     ///
     /// ```no_run
     /// use sysinfo::{System, SystemExt};
     ///
     /// let s = System::new_all();
-    /// println!("{} KB", s.free_swap());
+    /// println!("{} bytes", s.free_swap());
     /// ```
     fn free_swap(&self) -> u64;
 
-    /// Returns the amount of used SWAP in KB.
+    /// Returns the amount of used SWAP in bytes.
     ///
     /// ```no_run
     /// use sysinfo::{System, SystemExt};
     ///
     /// let s = System::new_all();
-    /// println!("{} KB", s.used_swap());
+    /// println!("{} bytes", s.used_swap());
     /// ```
     fn used_swap(&self) -> u64;
 
@@ -1011,18 +1127,6 @@ pub trait SystemExt: Sized + Debug + Default + Send + Sync {
     /// ```
     fn components_mut(&mut self) -> &mut [Component];
 
-    /// Returns the disks list.
-    ///
-    /// ```no_run
-    /// use sysinfo::{DiskExt, System, SystemExt};
-    ///
-    /// let s = System::new_all();
-    /// for disk in s.disks() {
-    ///     println!("{:?}", disk.name());
-    /// }
-    /// ```
-    fn disks(&self) -> &[Disk];
-
     /// Returns the users list.
     ///
     /// ```no_run
@@ -1035,6 +1139,18 @@ pub trait SystemExt: Sized + Debug + Default + Send + Sync {
     /// ```
     fn users(&self) -> &[User];
 
+    /// Returns the disks list.
+    ///
+    /// ```no_run
+    /// use sysinfo::{DiskExt, System, SystemExt};
+    ///
+    /// let s = System::new_all();
+    /// for disk in s.disks() {
+    ///     println!("{:?}", disk.name());
+    /// }
+    /// ```
+    fn disks(&self) -> &[Disk];
+
     /// Returns the disks list.
     ///
     /// ```no_run
@@ -1047,6 +1163,17 @@ pub trait SystemExt: Sized + Debug + Default + Send + Sync {
     /// ```
     fn disks_mut(&mut self) -> &mut [Disk];
 
+    /// Sort the disk list with the provided callback.
+    ///
+    /// Internally, it is using the [`slice::sort_unstable_by`] function, so please refer to it
+    /// for implementation details.
+    ///
+    /// ⚠️ If you use [`SystemExt::refresh_disks_list`], you need to use this method before using
+    /// [`SystemExt::disks`] or [`SystemExt::disks_mut`] if you want them to be sorted.
+    fn sort_disks_by<F>(&mut self, compare: F)
+    where
+        F: FnMut(&Disk, &Disk) -> std::cmp::Ordering;
+
     /// Returns the network interfaces object.
     ///
     /// ```no_run
@@ -1160,6 +1287,23 @@ pub trait SystemExt: Sized + Debug + Default + Send + Sync {
     /// ```
     fn long_os_version(&self) -> Option<String>;
 
+    /// Returns the distribution id as defined by os-release,
+    /// or [`std::env::consts::OS`].
+    ///
+    /// See also
+    /// - <https://www.freedesktop.org/software/systemd/man/os-release.html#ID=>
+    /// - <https://doc.rust-lang.org/std/env/consts/constant.OS.html>
+    ///
+    /// **Important**: this information is computed every time this function is called.
+    ///
+    /// ```no_run
+    /// use sysinfo::{System, SystemExt};
+    ///
+    /// let s = System::new();
+    /// println!("Distribution ID: {:?}", s.distribution_id());
+    /// ```
+    fn distribution_id(&self) -> String;
+
     /// Returns the system hostname based off DNS
     ///
     /// **Important**: this information is computed every time this function is called.
@@ -1171,6 +1315,35 @@ pub trait SystemExt: Sized + Debug + Default + Send + Sync {
     /// println!("Hostname: {:?}", s.host_name());
     /// ```
     fn host_name(&self) -> Option<String>;
+
+    /// Returns the [`User`] matching the given `user_id`.
+    ///
+    /// **Important**: The user list must be filled before using this method, otherwise it will
+    /// always return `None` (through the `refresh_*` methods).
+    ///
+    /// It is a shorthand for:
+    ///
+    /// ```ignore
+    /// let s = System::new_all();
+    /// s.users().find(|user| user.id() == user_id);
+    /// ```
+    ///
+    /// Full example:
+    ///
+    /// ```no_run
+    /// use sysinfo::{Pid, ProcessExt, System, SystemExt};
+    ///
+    /// let mut s = System::new_all();
+    ///
+    /// if let Some(process) = s.process(Pid::from(1337)) {
+    ///     if let Some(user_id) = process.user_id() {
+    ///         eprintln!("User for process 1337: {:?}", s.get_user_by_id(user_id));
+    ///     }
+    /// }
+    /// ```
+    fn get_user_by_id(&self, user_id: &Uid) -> Option<&User> {
+        self.users().iter().find(|user| user.id() == user_id)
+    }
 }
 
 /// Getting volume of received and transmitted data.
@@ -1382,10 +1555,17 @@ pub trait ComponentExt: Debug {
     ///     println!("{}°C", component.temperature());
     /// }
     /// ```
+    ///
+    /// ## Linux
+    ///
+    /// Returns `f32::NAN` if it failed to retrieve it.
     fn temperature(&self) -> f32;
 
     /// Returns the maximum temperature of the component (in celsius degree).
     ///
+    /// Note: if `temperature` is higher than the current `max`,
+    /// `max` value will be updated on refresh.
+    ///
     /// ```no_run
     /// use sysinfo::{ComponentExt, System, SystemExt};
     ///
@@ -1394,6 +1574,11 @@ pub trait ComponentExt: Debug {
     ///     println!("{}°C", component.max());
     /// }
     /// ```
+    ///
+    /// ## Linux
+    ///
+    /// May be computed by sysinfo from kernel.
+    /// Returns `f32::NAN` if it failed to retrieve it.
     fn max(&self) -> f32;
 
     /// Returns the highest temperature before the component halts (in celsius degree).
@@ -1406,6 +1591,10 @@ pub trait ComponentExt: Debug {
     ///     println!("{:?}°C", component.critical());
     /// }
     /// ```
+    ///
+    /// ## Linux
+    ///
+    /// Critical threshold defined by chip or kernel.
     fn critical(&self) -> Option<f32>;
 
     /// Returns the label of the component.
@@ -1418,6 +1607,19 @@ pub trait ComponentExt: Debug {
     ///     println!("{}", component.label());
     /// }
     /// ```
+    ///
+    /// ## Linux
+    ///
+    /// Since components informations are retrieved thanks to `hwmon`,
+    /// the labels are generated as follows.
+    /// Note: it may change and it was inspired by `sensors` own formatting.
+    ///
+    /// | name | label | device_model | id_sensor | Computed label by `sysinfo` |
+    /// |---------|--------|------------|----------|----------------------|
+    /// | ✓    | ✓    | ✓  | ✓ | `"{name} {label} {device_model} temp{id}"` |
+    /// | ✓    | ✓    | ✗  | ✓ | `"{name} {label} {id}"` |
+    /// | ✓    | ✗    | ✓  | ✓ | `"{name} {device_model}"` |
+    /// | ✓    | ✗    | ✗  | ✓ | `"{name} temp{id}"` |
     fn label(&self) -> &str;
 
     /// Refreshes component.
@@ -1453,10 +1655,10 @@ pub trait UserExt: Debug {
     ///
     /// let mut s = System::new_all();
     /// for user in s.users() {
-    ///     println!("{}", *user.uid());
+    ///     println!("{:?}", *user.id());
     /// }
     /// ```
-    fn uid(&self) -> Uid;
+    fn id(&self) -> &Uid;
 
     /// Return the group id of the user.
     ///
@@ -1470,10 +1672,10 @@ pub trait UserExt: Debug {
     ///
     /// let mut s = System::new_all();
     /// for user in s.users() {
-    ///     println!("{}", *user.gid());
+    ///     println!("{}", *user.group_id());
     /// }
     /// ```
-    fn gid(&self) -> Gid;
+    fn group_id(&self) -> Gid;
 
     /// Returns the name of the user.
     ///
