@@ -1,6 +1,10 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use crate::{DiskUsage, Pid, ProcessExt, ProcessStatus, Signal};
+use crate::sys::system::is_proc_running;
+use crate::sys::utils::to_str;
+use crate::{
+    DiskUsage, Gid, Pid, PidExt, ProcessExt, ProcessRefreshKind, ProcessStatus, Signal, Uid,
+};
 
 use std::ffi::OsString;
 use std::fmt;
@@ -12,6 +16,7 @@ use std::path::{Path, PathBuf};
 use std::process;
 use std::ptr::null_mut;
 use std::str;
+use std::sync::Arc;
 
 use libc::{c_void, memcpy};
 
@@ -30,17 +35,24 @@ use winapi::shared::ntdef::{NT_SUCCESS, UNICODE_STRING};
 use winapi::shared::ntstatus::{
     STATUS_BUFFER_OVERFLOW, STATUS_BUFFER_TOO_SMALL, STATUS_INFO_LENGTH_MISMATCH,
 };
+use winapi::shared::winerror::ERROR_INSUFFICIENT_BUFFER;
+use winapi::um::errhandlingapi::GetLastError;
 use winapi::um::handleapi::CloseHandle;
+use winapi::um::heapapi::{GetProcessHeap, HeapAlloc, HeapFree};
 use winapi::um::memoryapi::{ReadProcessMemory, VirtualQueryEx};
-use winapi::um::processthreadsapi::{GetProcessTimes, GetSystemTimes, OpenProcess};
+use winapi::um::processthreadsapi::{
+    GetProcessTimes, GetSystemTimes, OpenProcess, OpenProcessToken, ProcessIdToSessionId,
+};
 use winapi::um::psapi::{
     EnumProcessModulesEx, GetModuleBaseNameW, GetModuleFileNameExW, GetProcessMemoryInfo,
     LIST_MODULES_ALL, PROCESS_MEMORY_COUNTERS, PROCESS_MEMORY_COUNTERS_EX,
 };
+use winapi::um::securitybaseapi::GetTokenInformation;
 use winapi::um::winbase::{GetProcessIoCounters, CREATE_NO_WINDOW};
 use winapi::um::winnt::{
-    HANDLE, IO_COUNTERS, MEMORY_BASIC_INFORMATION, PROCESS_QUERY_INFORMATION, PROCESS_VM_READ,
-    RTL_OSVERSIONINFOEXW, ULARGE_INTEGER,
+    TokenUser, HANDLE, HEAP_ZERO_MEMORY, IO_COUNTERS, MEMORY_BASIC_INFORMATION,
+    PROCESS_QUERY_INFORMATION, PROCESS_QUERY_LIMITED_INFORMATION, PROCESS_VM_READ,
+    RTL_OSVERSIONINFOEXW, TOKEN_QUERY, TOKEN_USER, ULARGE_INTEGER,
 };
 
 impl fmt::Display for ProcessStatus {
@@ -52,36 +64,141 @@ impl fmt::Display for ProcessStatus {
     }
 }
 
-fn get_process_handler(pid: Pid) -> Option<HANDLE> {
+fn get_process_handler(pid: Pid) -> Option<HandleWrapper> {
     if pid.0 == 0 {
         return None;
     }
     let options = PROCESS_QUERY_INFORMATION | PROCESS_VM_READ;
 
-    unsafe {
-        let process_handler = OpenProcess(options, FALSE, pid.0 as DWORD);
-        if process_handler.is_null() {
+    HandleWrapper::new(unsafe { OpenProcess(options, FALSE, pid.0 as DWORD) })
+        .or_else(|| {
+            sysinfo_debug!("OpenProcess failed, error: {:?}", unsafe { GetLastError() });
+            HandleWrapper::new(unsafe {
+                OpenProcess(PROCESS_QUERY_LIMITED_INFORMATION, FALSE, pid.0 as DWORD)
+            })
+        })
+        .or_else(|| {
+            sysinfo_debug!("OpenProcess limited failed, error: {:?}", unsafe {
+                GetLastError()
+            });
+            None
+        })
+}
+
+unsafe fn get_process_user_id(
+    handle: &HandleWrapper,
+    refresh_kind: ProcessRefreshKind,
+) -> Option<Uid> {
+    struct HeapWrap<T>(*mut T);
+
+    impl<T> HeapWrap<T> {
+        unsafe fn new(size: DWORD) -> Option<Self> {
+            let ptr = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, size as _) as *mut T;
+            if ptr.is_null() {
+                sysinfo_debug!("HeapAlloc failed");
+                None
+            } else {
+                Some(Self(ptr))
+            }
+        }
+    }
+
+    impl<T> Drop for HeapWrap<T> {
+        fn drop(&mut self) {
+            if !self.0.is_null() {
+                unsafe {
+                    HeapFree(GetProcessHeap(), 0, self.0 as *mut _);
+                }
+            }
+        }
+    }
+
+    if !refresh_kind.user() {
+        return None;
+    }
+
+    let mut token = null_mut();
+
+    if OpenProcessToken(**handle, TOKEN_QUERY, &mut token) == 0 {
+        sysinfo_debug!("OpenProcessToken failed");
+        return None;
+    }
+
+    let token = HandleWrapper::new(token)?;
+
+    let mut size = 0;
+
+    if GetTokenInformation(*token, TokenUser, null_mut(), 0, &mut size) == 0 {
+        let err = GetLastError();
+        if err != ERROR_INSUFFICIENT_BUFFER {
+            sysinfo_debug!("GetTokenInformation failed, error: {:?}", err);
+            return None;
+        }
+    }
+
+    let ptu: HeapWrap<TOKEN_USER> = HeapWrap::new(size)?;
+
+    if GetTokenInformation(*token, TokenUser, ptu.0 as *mut _, size, &mut size) == 0 {
+        sysinfo_debug!("GetTokenInformation failed, error: {:?}", GetLastError());
+        return None;
+    }
+
+    let mut name_use = 0;
+    let mut name = [0u16; 256];
+    let mut domain_name = [0u16; 256];
+    let mut size = 256;
+
+    if winapi::um::winbase::LookupAccountSidW(
+        std::ptr::null_mut(),
+        (*ptu.0).User.Sid,
+        name.as_mut_ptr(),
+        &mut size,
+        domain_name.as_mut_ptr(),
+        &mut size,
+        &mut name_use,
+    ) == 0
+    {
+        sysinfo_debug!(
+            "LookupAccountSidW failed: {:?}",
+            winapi::um::errhandlingapi::GetLastError(),
+        );
+        None
+    } else {
+        Some(Uid(to_str(name.as_mut_ptr()).into_boxed_str()))
+    }
+}
+
+struct HandleWrapper(HANDLE);
+
+impl HandleWrapper {
+    fn new(handle: HANDLE) -> Option<Self> {
+        if handle.is_null() {
             None
         } else {
-            Some(process_handler)
+            Some(Self(handle))
         }
     }
 }
 
-#[derive(Clone)]
-struct PtrWrapper<T: Clone>(T);
-
-impl<T: Clone> Deref for PtrWrapper<T> {
-    type Target = T;
+impl Deref for HandleWrapper {
+    type Target = HANDLE;
 
     fn deref(&self) -> &Self::Target {
         &self.0
     }
 }
 
+impl Drop for HandleWrapper {
+    fn drop(&mut self) {
+        unsafe {
+            CloseHandle(self.0);
+        }
+    }
+}
+
 #[allow(clippy::non_send_fields_in_send_ty)]
-unsafe impl<T: Clone> Send for PtrWrapper<T> {}
-unsafe impl<T: Clone> Sync for PtrWrapper<T> {}
+unsafe impl Send for HandleWrapper {}
+unsafe impl Sync for HandleWrapper {}
 
 #[doc = include_str!("../../md_doc/process.md")]
 pub struct Process {
@@ -89,6 +206,7 @@ pub struct Process {
     cmd: Vec<String>,
     exe: PathBuf,
     pid: Pid,
+    user_id: Option<Uid>,
     environ: Vec<String>,
     cwd: PathBuf,
     root: PathBuf,
@@ -96,7 +214,7 @@ pub struct Process {
     pub(crate) virtual_memory: u64,
     parent: Option<Pid>,
     status: ProcessStatus,
-    handle: PtrWrapper<HANDLE>,
+    handle: Option<Arc<HandleWrapper>>,
     cpu_calc_values: CPUsageCalculationValues,
     start_time: u64,
     pub(crate) run_time: u64,
@@ -140,11 +258,11 @@ static WINDOWS_8_1_OR_NEWER: Lazy<bool> = Lazy::new(|| unsafe {
         || version_info.dwMajorVersion == 6 && version_info.dwMinorVersion >= 3
 });
 
-unsafe fn get_process_name(process_handler: HANDLE, h_mod: *mut c_void) -> String {
+unsafe fn get_process_name(process_handler: &HandleWrapper, h_mod: *mut c_void) -> String {
     let mut process_name = [0u16; MAX_PATH + 1];
 
     GetModuleBaseNameW(
-        process_handler,
+        **process_handler,
         h_mod as _,
         process_name.as_mut_ptr(),
         MAX_PATH as DWORD + 1,
@@ -152,10 +270,10 @@ unsafe fn get_process_name(process_handler: HANDLE, h_mod: *mut c_void) -> Strin
     null_terminated_wchar_to_string(&process_name)
 }
 
-unsafe fn get_h_mod(process_handler: HANDLE, h_mod: &mut *mut c_void) -> bool {
+unsafe fn get_h_mod(process_handler: &HandleWrapper, h_mod: &mut *mut c_void) -> bool {
     let mut cb_needed = 0;
     EnumProcessModulesEx(
-        process_handler,
+        **process_handler,
         h_mod as *mut *mut c_void as _,
         size_of::<DWORD>() as DWORD,
         &mut cb_needed,
@@ -163,11 +281,11 @@ unsafe fn get_h_mod(process_handler: HANDLE, h_mod: &mut *mut c_void) -> bool {
     ) != 0
 }
 
-unsafe fn get_exe(process_handler: HANDLE, h_mod: *mut c_void) -> PathBuf {
+unsafe fn get_exe(process_handler: &HandleWrapper) -> PathBuf {
     let mut exe_buf = [0u16; MAX_PATH + 1];
     GetModuleFileNameExW(
-        process_handler,
-        h_mod as _,
+        **process_handler,
+        std::ptr::null_mut(),
         exe_buf.as_mut_ptr(),
         MAX_PATH as DWORD + 1,
     );
@@ -176,51 +294,56 @@ unsafe fn get_exe(process_handler: HANDLE, h_mod: *mut c_void) -> PathBuf {
 }
 
 impl Process {
-    pub(crate) fn new_from_pid(pid: Pid, now: u64) -> Option<Process> {
+    pub(crate) fn new_from_pid(
+        pid: Pid,
+        now: u64,
+        refresh_kind: ProcessRefreshKind,
+    ) -> Option<Process> {
         unsafe {
             let process_handler = get_process_handler(pid)?;
             let mut info: MaybeUninit<PROCESS_BASIC_INFORMATION> = MaybeUninit::uninit();
             if NtQueryInformationProcess(
-                process_handler,
+                *process_handler,
                 ProcessBasicInformation,
                 info.as_mut_ptr() as *mut _,
                 size_of::<PROCESS_BASIC_INFORMATION>() as _,
                 null_mut(),
             ) != 0
             {
-                CloseHandle(process_handler);
                 return None;
             }
             let info = info.assume_init();
             let mut h_mod = null_mut();
 
-            let name = if get_h_mod(process_handler, &mut h_mod) {
-                get_process_name(process_handler, h_mod)
+            let name = if get_h_mod(&process_handler, &mut h_mod) {
+                get_process_name(&process_handler, h_mod)
             } else {
                 String::new()
             };
 
-            let exe = get_exe(process_handler, h_mod);
+            let exe = get_exe(&process_handler);
             let mut root = exe.clone();
             root.pop();
-            let (cmd, environ, cwd) = match get_process_params(process_handler) {
+            let (cmd, environ, cwd) = match get_process_params(&process_handler) {
                 Ok(args) => args,
                 Err(_e) => {
                     sysinfo_debug!("Failed to get process parameters: {}", _e);
                     (Vec::new(), Vec::new(), PathBuf::new())
                 }
             };
-            let (start_time, run_time) = get_start_and_run_time(process_handler, now);
+            let (start_time, run_time) = get_start_and_run_time(*process_handler, now);
             let parent = if info.InheritedFromUniqueProcessId as usize != 0 {
                 Some(Pid(info.InheritedFromUniqueProcessId as _))
             } else {
                 None
             };
+            let user_id = get_process_user_id(&process_handler, refresh_kind);
             Some(Process {
-                handle: PtrWrapper(process_handler),
+                handle: Some(Arc::new(process_handler)),
                 name,
                 pid,
                 parent,
+                user_id,
                 cmd,
                 environ,
                 exe,
@@ -249,30 +372,27 @@ impl Process {
         virtual_memory: u64,
         name: String,
         now: u64,
+        refresh_kind: ProcessRefreshKind,
     ) -> Process {
         if let Some(handle) = get_process_handler(pid) {
-            let mut h_mod = null_mut();
-
             unsafe {
-                let exe = if get_h_mod(handle, &mut h_mod) {
-                    get_exe(handle, h_mod)
-                } else {
-                    PathBuf::new()
-                };
+                let exe = get_exe(&handle);
                 let mut root = exe.clone();
                 root.pop();
-                let (cmd, environ, cwd) = match get_process_params(handle) {
+                let (cmd, environ, cwd) = match get_process_params(&handle) {
                     Ok(args) => args,
                     Err(_e) => {
                         sysinfo_debug!("Failed to get process parameters: {}", _e);
                         (Vec::new(), Vec::new(), PathBuf::new())
                     }
                 };
-                let (start_time, run_time) = get_start_and_run_time(handle, now);
+                let (start_time, run_time) = get_start_and_run_time(*handle, now);
+                let user_id = get_process_user_id(&handle, refresh_kind);
                 Process {
-                    handle: PtrWrapper(handle),
+                    handle: Some(Arc::new(handle)),
                     name,
                     pid,
+                    user_id,
                     parent,
                     cmd,
                     environ,
@@ -295,9 +415,10 @@ impl Process {
             }
         } else {
             Process {
-                handle: PtrWrapper(null_mut()),
+                handle: None,
                 name,
                 pid,
+                user_id: None,
                 parent,
                 cmd: Vec::new(),
                 environ: Vec::new(),
@@ -323,11 +444,11 @@ impl Process {
     pub(crate) fn update(
         &mut self,
         refresh_kind: crate::ProcessRefreshKind,
-        nb_processors: u64,
+        nb_cpus: u64,
         now: u64,
     ) {
         if refresh_kind.cpu() {
-            compute_cpu_usage(self, nb_processors);
+            compute_cpu_usage(self, nb_cpus);
         }
         if refresh_kind.disk_usage() {
             update_disk_usage(self);
@@ -335,11 +456,19 @@ impl Process {
         self.run_time = now.saturating_sub(self.start_time());
         self.updated = true;
     }
+
+    pub(crate) fn get_handle(&self) -> Option<HANDLE> {
+        self.handle.as_ref().map(|h| ***h)
+    }
+
+    pub(crate) fn get_start_time(&self) -> Option<u64> {
+        self.handle.as_ref().map(|handle| get_start_time(***handle))
+    }
 }
 
 impl ProcessExt for Process {
     fn kill_with(&self, signal: Signal) -> Option<bool> {
-        let _c_signal = super::system::convert_signal(signal)?;
+        super::system::convert_signal(signal)?;
         let mut kill = process::Command::new("taskkill.exe");
         kill.arg("/PID").arg(self.pid.to_string()).arg("/F");
         kill.creation_flags(CREATE_NO_WINDOW);
@@ -413,20 +542,44 @@ impl ProcessExt for Process {
             total_read_bytes: self.read_bytes,
         }
     }
-}
 
-impl Drop for Process {
-    fn drop(&mut self) {
-        unsafe {
-            if self.handle.is_null() {
-                return;
+    fn user_id(&self) -> Option<&Uid> {
+        self.user_id.as_ref()
+    }
+
+    fn group_id(&self) -> Option<Gid> {
+        None
+    }
+
+    fn wait(&self) {
+        if let Some(handle) = self.get_handle() {
+            while is_proc_running(handle) {
+                if get_start_time(handle) != self.start_time() {
+                    // PID owner changed so the previous process was finished!
+                    return;
+                }
+                std::thread::sleep(std::time::Duration::from_millis(10));
             }
-            CloseHandle(*self.handle);
+        } else {
+            // In this case, we can't do anything so we just return.
+            sysinfo_debug!("can't wait on this process so returning");
+        }
+    }
+
+    fn session_id(&self) -> Option<Pid> {
+        unsafe {
+            let mut out = 0;
+            if ProcessIdToSessionId(self.pid.as_u32(), &mut out) != 0 {
+                return Some(Pid(out as _));
+            }
+            sysinfo_debug!("ProcessIdToSessionId failed, error: {:?}", GetLastError());
+            None
         }
     }
 }
 
-unsafe fn get_start_and_run_time(handle: HANDLE, now: u64) -> (u64, u64) {
+#[inline]
+unsafe fn get_process_times(handle: HANDLE) -> u64 {
     let mut fstart: FILETIME = zeroed();
     let mut x = zeroed();
 
@@ -437,25 +590,44 @@ unsafe fn get_start_and_run_time(handle: HANDLE, now: u64) -> (u64, u64) {
         &mut x as *mut FILETIME,
         &mut x as *mut FILETIME,
     );
-    let tmp = super::utils::filetime_to_u64(fstart);
+    super::utils::filetime_to_u64(fstart)
+}
+
+#[inline]
+fn compute_start(process_times: u64) -> u64 {
     // 11_644_473_600 is the number of seconds between the Windows epoch (1601-01-01) and
     // the linux epoch (1970-01-01).
-    let start = tmp / 10_000_000 - 11_644_473_600;
-    let run_time = check_sub(now, start);
-    (start, run_time)
+    process_times / 10_000_000 - 11_644_473_600
+}
+
+fn get_start_and_run_time(handle: HANDLE, now: u64) -> (u64, u64) {
+    unsafe {
+        let process_times = get_process_times(handle);
+        let start = compute_start(process_times);
+        let run_time = check_sub(now, start);
+        (start, run_time)
+    }
+}
+
+#[inline]
+pub(crate) fn get_start_time(handle: HANDLE) -> u64 {
+    unsafe {
+        let process_times = get_process_times(handle);
+        compute_start(process_times)
+    }
 }
 
 #[allow(clippy::uninit_vec)]
 unsafe fn ph_query_process_variable_size(
-    process_handle: HANDLE,
+    process_handle: &HandleWrapper,
     process_information_class: PROCESSINFOCLASS,
 ) -> Option<Vec<u16>> {
     let mut return_length = MaybeUninit::<ULONG>::uninit();
 
     let mut status = NtQueryInformationProcess(
-        process_handle,
+        **process_handle,
         process_information_class,
-        std::ptr::null_mut(),
+        null_mut(),
         0,
         return_length.as_mut_ptr() as *mut _,
     );
@@ -473,7 +645,7 @@ unsafe fn ph_query_process_variable_size(
     buffer.set_len(buf_len);
 
     status = NtQueryInformationProcess(
-        process_handle,
+        **process_handle,
         process_information_class,
         buffer.as_mut_ptr() as *mut _,
         return_length,
@@ -508,10 +680,10 @@ unsafe fn get_cmdline_from_buffer(buffer: *const u16) -> Vec<String> {
     res
 }
 
-unsafe fn get_region_size(handle: HANDLE, ptr: LPVOID) -> Result<usize, &'static str> {
+unsafe fn get_region_size(handle: &HandleWrapper, ptr: LPVOID) -> Result<usize, &'static str> {
     let mut meminfo = MaybeUninit::<MEMORY_BASIC_INFORMATION>::uninit();
     if VirtualQueryEx(
-        handle,
+        **handle,
         ptr,
         meminfo.as_mut_ptr() as *mut _,
         size_of::<MEMORY_BASIC_INFORMATION>(),
@@ -525,18 +697,18 @@ unsafe fn get_region_size(handle: HANDLE, ptr: LPVOID) -> Result<usize, &'static
 
 #[allow(clippy::uninit_vec)]
 unsafe fn get_process_data(
-    handle: HANDLE,
+    handle: &HandleWrapper,
     ptr: LPVOID,
     size: usize,
 ) -> Result<Vec<u16>, &'static str> {
     let mut buffer: Vec<u16> = Vec::with_capacity(size / 2 + 1);
     buffer.set_len(size / 2);
     if ReadProcessMemory(
-        handle,
+        **handle,
         ptr as *mut _,
         buffer.as_mut_ptr() as *mut _,
         size,
-        std::ptr::null_mut(),
+        null_mut(),
     ) != TRUE
     {
         return Err("Unable to read process data");
@@ -545,25 +717,25 @@ unsafe fn get_process_data(
 }
 
 trait RtlUserProcessParameters {
-    fn get_cmdline(&self, handle: HANDLE) -> Result<Vec<u16>, &'static str>;
-    fn get_cwd(&self, handle: HANDLE) -> Result<Vec<u16>, &'static str>;
-    fn get_environ(&self, handle: HANDLE) -> Result<Vec<u16>, &'static str>;
+    fn get_cmdline(&self, handle: &HandleWrapper) -> Result<Vec<u16>, &'static str>;
+    fn get_cwd(&self, handle: &HandleWrapper) -> Result<Vec<u16>, &'static str>;
+    fn get_environ(&self, handle: &HandleWrapper) -> Result<Vec<u16>, &'static str>;
 }
 
 macro_rules! impl_RtlUserProcessParameters {
     ($t:ty) => {
         impl RtlUserProcessParameters for $t {
-            fn get_cmdline(&self, handle: HANDLE) -> Result<Vec<u16>, &'static str> {
+            fn get_cmdline(&self, handle: &HandleWrapper) -> Result<Vec<u16>, &'static str> {
                 let ptr = self.CommandLine.Buffer;
                 let size = self.CommandLine.Length;
                 unsafe { get_process_data(handle, ptr as _, size as _) }
             }
-            fn get_cwd(&self, handle: HANDLE) -> Result<Vec<u16>, &'static str> {
+            fn get_cwd(&self, handle: &HandleWrapper) -> Result<Vec<u16>, &'static str> {
                 let ptr = self.CurrentDirectory.DosPath.Buffer;
                 let size = self.CurrentDirectory.DosPath.Length;
                 unsafe { get_process_data(handle, ptr as _, size as _) }
             }
-            fn get_environ(&self, handle: HANDLE) -> Result<Vec<u16>, &'static str> {
+            fn get_environ(&self, handle: &HandleWrapper) -> Result<Vec<u16>, &'static str> {
                 let ptr = self.Environment;
                 unsafe {
                     let size = get_region_size(handle, ptr as LPVOID)?;
@@ -578,7 +750,7 @@ impl_RtlUserProcessParameters!(RTL_USER_PROCESS_PARAMETERS32);
 impl_RtlUserProcessParameters!(RTL_USER_PROCESS_PARAMETERS);
 
 unsafe fn get_process_params(
-    handle: HANDLE,
+    handle: &HandleWrapper,
 ) -> Result<(Vec<String>, Vec<String>, PathBuf), &'static str> {
     if !cfg!(target_pointer_width = "64") {
         return Err("Non 64 bit targets are not supported");
@@ -587,7 +759,7 @@ unsafe fn get_process_params(
     // First check if target process is running in wow64 compatibility emulator
     let mut pwow32info = MaybeUninit::<LPVOID>::uninit();
     let result = NtQueryInformationProcess(
-        handle,
+        **handle,
         ProcessWow64Information,
         pwow32info.as_mut_ptr() as *mut _,
         size_of::<LPVOID>() as u32,
@@ -603,7 +775,7 @@ unsafe fn get_process_params(
 
         let mut pbasicinfo = MaybeUninit::<PROCESS_BASIC_INFORMATION>::uninit();
         let result = NtQueryInformationProcess(
-            handle,
+            **handle,
             ProcessBasicInformation,
             pbasicinfo.as_mut_ptr() as *mut _,
             size_of::<PROCESS_BASIC_INFORMATION>() as u32,
@@ -616,11 +788,11 @@ unsafe fn get_process_params(
 
         let mut peb = MaybeUninit::<PEB>::uninit();
         if ReadProcessMemory(
-            handle,
+            **handle,
             pinfo.PebBaseAddress as *mut _,
             peb.as_mut_ptr() as *mut _,
             size_of::<PEB>() as SIZE_T,
-            std::ptr::null_mut(),
+            null_mut(),
         ) != TRUE
         {
             return Err("Unable to read process PEB");
@@ -630,11 +802,11 @@ unsafe fn get_process_params(
 
         let mut proc_params = MaybeUninit::<RTL_USER_PROCESS_PARAMETERS>::uninit();
         if ReadProcessMemory(
-            handle,
+            **handle,
             peb.ProcessParameters as *mut PRTL_USER_PROCESS_PARAMETERS as *mut _,
             proc_params.as_mut_ptr() as *mut _,
             size_of::<RTL_USER_PROCESS_PARAMETERS>() as SIZE_T,
-            std::ptr::null_mut(),
+            null_mut(),
         ) != TRUE
         {
             return Err("Unable to read process parameters");
@@ -651,11 +823,11 @@ unsafe fn get_process_params(
 
     let mut peb32 = MaybeUninit::<PEB32>::uninit();
     if ReadProcessMemory(
-        handle,
+        **handle,
         pwow32info,
         peb32.as_mut_ptr() as *mut _,
         size_of::<PEB32>() as SIZE_T,
-        std::ptr::null_mut(),
+        null_mut(),
     ) != TRUE
     {
         return Err("Unable to read PEB32");
@@ -664,11 +836,11 @@ unsafe fn get_process_params(
 
     let mut proc_params = MaybeUninit::<RTL_USER_PROCESS_PARAMETERS32>::uninit();
     if ReadProcessMemory(
-        handle,
+        **handle,
         peb32.ProcessParameters as *mut PRTL_USER_PROCESS_PARAMETERS32 as *mut _,
         proc_params.as_mut_ptr() as *mut _,
         size_of::<RTL_USER_PROCESS_PARAMETERS32>() as SIZE_T,
-        std::ptr::null_mut(),
+        null_mut(),
     ) != TRUE
     {
         return Err("Unable to read 32 bit process parameters");
@@ -681,7 +853,7 @@ unsafe fn get_process_params(
     ))
 }
 
-fn get_cwd<T: RtlUserProcessParameters>(params: &T, handle: HANDLE) -> PathBuf {
+fn get_cwd<T: RtlUserProcessParameters>(params: &T, handle: &HandleWrapper) -> PathBuf {
     match params.get_cwd(handle) {
         Ok(buffer) => unsafe { PathBuf::from(null_terminated_wchar_to_string(buffer.as_slice())) },
         Err(_e) => {
@@ -700,7 +872,10 @@ unsafe fn null_terminated_wchar_to_string(slice: &[u16]) -> String {
     }
 }
 
-fn get_cmd_line_old<T: RtlUserProcessParameters>(params: &T, handle: HANDLE) -> Vec<String> {
+fn get_cmd_line_old<T: RtlUserProcessParameters>(
+    params: &T,
+    handle: &HandleWrapper,
+) -> Vec<String> {
     match params.get_cmdline(handle) {
         Ok(buffer) => unsafe { get_cmdline_from_buffer(buffer.as_ptr()) },
         Err(_e) => {
@@ -711,7 +886,7 @@ fn get_cmd_line_old<T: RtlUserProcessParameters>(params: &T, handle: HANDLE) ->
 }
 
 #[allow(clippy::cast_ptr_alignment)]
-fn get_cmd_line_new(handle: HANDLE) -> Vec<String> {
+fn get_cmd_line_new(handle: &HandleWrapper) -> Vec<String> {
     unsafe {
         if let Some(buffer) = ph_query_process_variable_size(handle, ProcessCommandLineInformation)
         {
@@ -724,7 +899,7 @@ fn get_cmd_line_new(handle: HANDLE) -> Vec<String> {
     }
 }
 
-fn get_cmd_line<T: RtlUserProcessParameters>(params: &T, handle: HANDLE) -> Vec<String> {
+fn get_cmd_line<T: RtlUserProcessParameters>(params: &T, handle: &HandleWrapper) -> Vec<String> {
     if *WINDOWS_8_1_OR_NEWER {
         get_cmd_line_new(handle)
     } else {
@@ -732,7 +907,7 @@ fn get_cmd_line<T: RtlUserProcessParameters>(params: &T, handle: HANDLE) -> Vec<
     }
 }
 
-fn get_proc_env<T: RtlUserProcessParameters>(params: &T, handle: HANDLE) -> Vec<String> {
+fn get_proc_env<T: RtlUserProcessParameters>(params: &T, handle: &HandleWrapper) -> Vec<String> {
     match params.get_environ(handle) {
         Ok(buffer) => {
             let equals = "=".encode_utf16().next().unwrap();
@@ -786,7 +961,7 @@ fn check_sub(a: u64, b: u64) -> u64 {
 
 /// Before changing this function, you must consider the following:
 /// https://github.com/GuillaumeGomez/sysinfo/issues/459
-pub(crate) fn compute_cpu_usage(p: &mut Process, nb_processors: u64) {
+pub(crate) fn compute_cpu_usage(p: &mut Process, nb_cpus: u64) {
     unsafe {
         let mut ftime: FILETIME = zeroed();
         let mut fsys: FILETIME = zeroed();
@@ -795,13 +970,15 @@ pub(crate) fn compute_cpu_usage(p: &mut Process, nb_processors: u64) {
         let mut fglobal_kernel_time: FILETIME = zeroed(); // notice that it includes idle time
         let mut fglobal_user_time: FILETIME = zeroed();
 
-        GetProcessTimes(
-            *p.handle,
-            &mut ftime as *mut FILETIME,
-            &mut ftime as *mut FILETIME,
-            &mut fsys as *mut FILETIME,
-            &mut fuser as *mut FILETIME,
-        );
+        if let Some(handle) = p.get_handle() {
+            GetProcessTimes(
+                handle,
+                &mut ftime as *mut FILETIME,
+                &mut ftime as *mut FILETIME,
+                &mut fsys as *mut FILETIME,
+                &mut fuser as *mut FILETIME,
+            );
+        }
         GetSystemTimes(
             &mut fglobal_idle_time as *mut FILETIME,
             &mut fglobal_kernel_time as *mut FILETIME,
@@ -845,57 +1022,57 @@ pub(crate) fn compute_cpu_usage(p: &mut Process, nb_processors: u64) {
         let delta_user_time = check_sub(user, p.cpu_calc_values.old_process_user_cpu);
         let delta_sys_time = check_sub(sys, p.cpu_calc_values.old_process_sys_cpu);
 
-        let denominator = (delta_global_user_time + delta_global_kernel_time) as f64;
-
-        p.cpu_usage = 100.0
-            * ((delta_user_time + delta_sys_time) as f64
-                / if denominator == 0.0 {
-                    p.cpu_usage = 0.0;
-                    return;
-                } else {
-                    denominator
-                }) as f32
-            * nb_processors as f32;
         p.cpu_calc_values.old_process_user_cpu = user;
         p.cpu_calc_values.old_process_sys_cpu = sys;
         p.cpu_calc_values.old_system_user_cpu = global_user_time;
         p.cpu_calc_values.old_system_sys_cpu = global_kernel_time;
-    }
-}
 
-pub(crate) fn get_handle(p: &Process) -> HANDLE {
-    *p.handle
+        let denominator = delta_global_user_time.saturating_add(delta_global_kernel_time) as f32;
+
+        if denominator < 0.00001 {
+            p.cpu_usage = 0.;
+            return;
+        }
+
+        p.cpu_usage = 100.0
+            * (delta_user_time.saturating_add(delta_sys_time) as f32 / denominator)
+            * nb_cpus as f32;
+    }
 }
 
 pub(crate) fn update_disk_usage(p: &mut Process) {
     let mut counters = MaybeUninit::<IO_COUNTERS>::uninit();
 
-    unsafe {
-        let ret = GetProcessIoCounters(*p.handle, counters.as_mut_ptr());
-        if ret == 0 {
-            sysinfo_debug!("GetProcessIoCounters call failed on process {}", p.pid());
-        } else {
-            let counters = counters.assume_init();
-            p.old_read_bytes = p.read_bytes;
-            p.old_written_bytes = p.written_bytes;
-            p.read_bytes = counters.ReadTransferCount;
-            p.written_bytes = counters.WriteTransferCount;
+    if let Some(handle) = p.get_handle() {
+        unsafe {
+            let ret = GetProcessIoCounters(handle, counters.as_mut_ptr());
+            if ret == 0 {
+                sysinfo_debug!("GetProcessIoCounters call failed on process {}", p.pid());
+            } else {
+                let counters = counters.assume_init();
+                p.old_read_bytes = p.read_bytes;
+                p.old_written_bytes = p.written_bytes;
+                p.read_bytes = counters.ReadTransferCount;
+                p.written_bytes = counters.WriteTransferCount;
+            }
         }
     }
 }
 
 pub(crate) fn update_memory(p: &mut Process) {
-    unsafe {
-        let mut pmc: PROCESS_MEMORY_COUNTERS_EX = zeroed();
-        if GetProcessMemoryInfo(
-            *p.handle,
-            &mut pmc as *mut PROCESS_MEMORY_COUNTERS_EX as *mut c_void
-                as *mut PROCESS_MEMORY_COUNTERS,
-            size_of::<PROCESS_MEMORY_COUNTERS_EX>() as DWORD,
-        ) != 0
-        {
-            p.memory = (pmc.WorkingSetSize as u64) / 1_000;
-            p.virtual_memory = (pmc.PrivateUsage as u64) / 1_000;
+    if let Some(handle) = p.get_handle() {
+        unsafe {
+            let mut pmc: PROCESS_MEMORY_COUNTERS_EX = zeroed();
+            if GetProcessMemoryInfo(
+                handle,
+                &mut pmc as *mut PROCESS_MEMORY_COUNTERS_EX as *mut c_void
+                    as *mut PROCESS_MEMORY_COUNTERS,
+                size_of::<PROCESS_MEMORY_COUNTERS_EX>() as DWORD,
+            ) != 0
+            {
+                p.memory = pmc.WorkingSetSize as _;
+                p.virtual_memory = pmc.PrivateUsage as _;
+            }
         }
     }
 }
